@@ -9,7 +9,9 @@
 #ifndef NFSLIB_H
 #define NFSLIB_H
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -23,6 +25,9 @@
 #ifndef _PATH_EXPORTS
 #define _PATH_EXPORTS		"/etc/exports"
 #endif
+#ifndef _PATH_IDMAPDCONF
+#define _PATH_IDMAPDCONF	"/etc/idmapd.conf"
+#endif
 #ifndef _PATH_XTAB
 #define _PATH_XTAB		NFS_STATEDIR "/xtab"
 #endif
@@ -43,6 +48,7 @@
 #endif
 #ifndef _PATH_PROC_EXPORTS
 #define	_PATH_PROC_EXPORTS	"/proc/fs/nfs/exports"
+#define	_PATH_PROC_EXPORTS_ALT	"/proc/fs/nfsd/exports"
 #endif
 
 enum cle_maptypes {
@@ -53,7 +59,7 @@ enum cle_maptypes {
 
 /*
  * Data related to a single exports entry as returned by getexportent.
- * FIXME: export options should probably be parsed at a later time to 
+ * FIXME: export options should probably be parsed at a later time to
  * allow overrides when using exportfs.
  */
 struct exportent {
@@ -73,6 +79,7 @@ struct exportent {
 	int *		e_sqgids;
 	int		e_nsqgids;
 	int		e_fsid;
+	char *		e_mountpoint;
 };
 
 struct rmtabent {
@@ -126,6 +133,13 @@ void qword_print(FILE *f, char *str);
 void qword_printhex(FILE *f, char *str, int slen);
 void qword_printint(FILE *f, int num);
 void qword_eol(FILE *f);
+int readline(int fd, char **buf, int *lenp);
+int qword_get(char **bpp, char *dest, int bufsize);
+int qword_get_int(char **bpp, int *anint);
+void cache_flush(int force);
+int check_new_cache(void);
+
+void closeall(int min);
 
 /* lockd. */
 int			lockdsvc();
