@@ -14,7 +14,9 @@
  *		as is, with no warranty expressed or implied.
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/param.h>
 #include <stdlib.h>
@@ -67,7 +69,7 @@ struct exportent *
 getexportent(int fromkernel, int fromexports)
 {
 	static struct exportent	ee;
-	char		exp[512];
+	char		exp[512], *hostname;
 	char		rpath[MAXPATHLEN+1];
 	char		*opt, *sp;
 	int		ok;
@@ -91,6 +93,7 @@ getexportent(int fromkernel, int fromexports)
 	ee.e_anongid = -2;
 	ee.e_squids = NULL;
 	ee.e_sqgids = NULL;
+	ee.e_mountpoint = NULL;
 	ee.e_nsquids = 0;
 	ee.e_nsqgids = 0;
 
@@ -112,9 +115,13 @@ getexportent(int fromkernel, int fromexports)
 	/* Check for default client */
 	if (ok == 0)
 		exp[0] = '\0';
+
+	hostname = exp;
 	if ((opt = strchr(exp, '(')) != NULL) {
-		if (opt == exp) 
+		if (opt == exp) {
 			xlog(L_WARNING, "No host name given with %s %s, suggest *%s to avoid warning", ee.e_path, exp, exp);
+			hostname = "*";
+		}
 		*opt++ = '\0';
 		if (!(sp = strchr(opt, ')')) || sp[1] != '\0') {
 			syntaxerr("bad option list");
@@ -125,12 +132,12 @@ getexportent(int fromkernel, int fromexports)
 	} else {
 	    xlog(L_WARNING, "No options for %s %s: suggest %s(sync) to avoid warning", ee.e_path, exp, exp);
 	}
-	if (strlen(exp) >= sizeof(ee.e_hostname)) {
+	if (strlen(hostname) >= sizeof(ee.e_hostname)) {
 		syntaxerr("client name too long");
 		export_errno = EINVAL;
 		return NULL;
 	}
-	strncpy(ee.e_hostname, exp, sizeof (ee.e_hostname) - 1);
+	strncpy(ee.e_hostname, hostname, sizeof (ee.e_hostname) - 1);
 	ee.e_hostname[sizeof (ee.e_hostname) - 1] = '\0';
 
 	if (parseopts(opt, &ee, fromexports) < 0)
@@ -170,8 +177,10 @@ putexportent(struct exportent *ep)
 	fprintf(fp, "%ssync,", (ep->e_flags & NFSEXP_ASYNC)? "a" : "");
 	fprintf(fp, "%swdelay,", (ep->e_flags & NFSEXP_GATHERED_WRITES)?
 				"" : "no_");
-	fprintf(fp, "%shide,", (ep->e_flags & NFSEXP_CROSSMNT)?
+	fprintf(fp, "%shide,", (ep->e_flags & NFSEXP_NOHIDE)?
 				"no" : "");
+	fprintf(fp, "%scrossmnt,", (ep->e_flags & NFSEXP_CROSSMOUNT)?
+				"" : "no");
 	fprintf(fp, "%ssecure,", (ep->e_flags & NFSEXP_INSECURE_PORT)?
 				"in" : "");
 	fprintf(fp, "%sroot_squash,", (ep->e_flags & NFSEXP_ROOTSQUASH)?
@@ -182,9 +191,15 @@ putexportent(struct exportent *ep)
 		"no_" : "");
 	fprintf(fp, "%ssecure_locks,", (ep->e_flags & NFSEXP_NOAUTHNLM)?
 		"in" : "");
+	fprintf(fp, "%sacl,", (ep->e_flags & NFSEXP_NOACL)?
+		"no_" : "");
 	if (ep->e_flags & NFSEXP_FSID) {
 		fprintf(fp, "fsid=%d,", ep->e_fsid);
 	}
+	if (ep->e_mountpoint)
+		fprintf(fp, "mountpoint%s%s,",
+			ep->e_mountpoint[0]?"=":"", ep->e_mountpoint);
+
 	fprintf(fp, "mapping=");
 	switch (ep->e_maptype) {
 	case CLE_MAP_IDENT:
@@ -245,6 +260,8 @@ dupexportent(struct exportent *dst, struct exportent *src)
 		dst->e_sqgids = (int *) xmalloc(n * sizeof(int));
 		memcpy(dst->e_sqgids, src->e_sqgids, n * sizeof(int));
 	}
+	if (src->e_mountpoint)
+		dst->e_mountpoint = strdup(src->e_mountpoint);
 }
 
 struct exportent *
@@ -258,6 +275,7 @@ mkexportent(char *hname, char *path, char *options)
 	ee.e_anongid = -2;
 	ee.e_squids = NULL;
 	ee.e_sqgids = NULL;
+	ee.e_mountpoint = NULL;
 	ee.e_nsquids = 0;
 	ee.e_nsqgids = 0;
 
@@ -333,13 +351,13 @@ parseopts(char *cp, struct exportent *ep, int warn)
 			had_sync_opt = 1;
 			ep->e_flags |= NFSEXP_ASYNC;
 		} else if (!strcmp(opt, "nohide"))
-			ep->e_flags |= NFSEXP_CROSSMNT;
+			ep->e_flags |= NFSEXP_NOHIDE;
 		else if (!strcmp(opt, "hide"))
-			ep->e_flags &= ~NFSEXP_CROSSMNT;
-		else if (!strcmp(opt, "crossmnt"))		/* old style */
-			ep->e_flags |= NFSEXP_CROSSMNT;
-		else if (!strcmp(opt, "nocrossmnt"))		/* old style */
-			ep->e_flags &= ~NFSEXP_CROSSMNT;
+			ep->e_flags &= ~NFSEXP_NOHIDE;
+		else if (!strcmp(opt, "crossmnt"))
+			ep->e_flags |= NFSEXP_CROSSMOUNT;
+		else if (!strcmp(opt, "nocrossmnt"))
+			ep->e_flags &= ~NFSEXP_CROSSMOUNT;
 		else if (!strcmp(opt, "wdelay"))
 			ep->e_flags |= NFSEXP_GATHERED_WRITES;
 		else if (!strcmp(opt, "no_wdelay"))
@@ -364,6 +382,10 @@ parseopts(char *cp, struct exportent *ep, int warn)
 			ep->e_flags &= ~NFSEXP_NOAUTHNLM;
 		else if (strcmp(opt, "insecure_locks") == 0)
 			ep->e_flags |= NFSEXP_NOAUTHNLM;
+		else if (strcmp(opt, "acl") == 0)
+			ep->e_flags &= ~NFSEXP_NOACL;
+		else if (strcmp(opt, "no_acl") == 0)
+			ep->e_flags |= NFSEXP_NOACL;
 		else if (strncmp(opt, "mapping=", 8) == 0)
 			ep->e_maptype = parsemaptype(opt+8);
 		else if (strcmp(opt, "map_identity") == 0)	/* old style */
@@ -406,6 +428,15 @@ bad_option:
 				goto bad_option;
 			}
 			ep->e_flags |= NFSEXP_FSID;
+		} else if (strcmp(opt, "mountpoint")==0 ||
+			   strcmp(opt, "mp") == 0 ||
+			   strncmp(opt, "mountpoint=", 11)==0 ||
+			   strncmp(opt, "mp=", 3) == 0) {
+			char * mp = strchr(opt, '=');
+			if (mp)
+				ep->e_mountpoint = strdup(mp+1);
+			else
+				ep->e_mountpoint = strdup("");
 		} else {
 			xlog(L_ERROR, "%s:%d: unknown keyword \"%s\"\n",
 					flname, flline, opt);
@@ -423,7 +454,7 @@ bad_option:
 	ep->e_nsqgids = nsqgids;
 
 out:
-	if (warn && !had_sync_opt)
+	if (warn && !had_sync_opt && !(ep->e_flags & NFSEXP_READONLY))
 		xlog(L_WARNING, "%s [%d]: No 'sync' or 'async' option specified for export \"%s:%s\".\n"
 				"  Assuming default behaviour ('sync').\n"
 		     		"  NOTE: this default has changed from previous versions\n",
