@@ -6,7 +6,9 @@
  * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <unistd.h>
 #include <fcntl.h>
@@ -19,7 +21,9 @@ nfssvc(int port, int nrservs)
 	struct nfsctl_arg	arg;
 	int fd;
 
-	fd = open("/proc/fs/nfs/threads", O_WRONLY);
+	fd = open("/proc/fs/nfsd/threads", O_WRONLY);
+	if (fd < 0)
+		fd = open("/proc/fs/nfs/threads", O_WRONLY);
 	if (fd >= 0) {
 		/* 2.5+ kernel with nfsd filesystem mounted.
 		 * Just write the number in.
