@@ -6,7 +6,9 @@
  * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/fcntl.h>
 #include <string.h>
@@ -160,7 +162,7 @@ xskip(XFILE *xfp, char *str)
 	int	c;
 
 	while ((c = xgetc(xfp)) != EOF && strchr(str, c));
-	ungetc(c, xfp->x_fp);
+	xungetc(c, xfp);
 }
 
 char
