@@ -6,7 +6,9 @@
  * Copyright (C) 1995, 1996 Olaf Kirch <okir@monad.swb.de>
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <signal.h>
 #include <sys/stat.h>
@@ -36,6 +38,11 @@ static struct nfs_fh_len *get_rootfh(struct svc_req *, dirpath *, int *, int v3)
 
 int new_cache = 0;
 
+/* PRC: a high-availability callout program can be specified with -H
+ * When this is done, the program will receive callouts whenever clients
+ * send mount or unmount requests -- the callout is not needed for 2.6 kernel */
+char *ha_callout_prog = NULL;
+
 static struct option longopts[] =
 {
 	{ "foreground", 0, 0, 'F' },
@@ -48,6 +55,7 @@ static struct option longopts[] =
 	{ "version", 0, 0, 'v' },
 	{ "port", 1, 0, 'p' },
 	{ "no-tcp", 0, 0, 'n' },
+	{ "ha-callout", 1, 0, 'H' },
 	{ NULL, 0, 0, 0 }
 };
 
@@ -68,6 +76,14 @@ killer (int sig)
   xlog (L_FATAL, "Caught signal %d, un-registering and exiting.", sig);
 }
 
+static void
+sig_hup (int sig)
+{
+  /* don't exit on SIGHUP */
+  xlog (L_NOTICE, "Received SIGHUP... Ignoring.\n", sig);
+  return;
+}
+
 bool_t
 mount_null_1_svc(struct svc_req *rqstp, void *argp, void *resp)
 {
@@ -90,10 +106,11 @@ mount_dump_1_svc(struct svc_req *rqstp, void *argp, mountlist *res)
 {
 	struct sockaddr_in *addr =
 		(struct sockaddr_in *) svc_getcaller(rqstp->rq_xprt);
-	xlog(L_NOTICE, "dump request from %s",
-		inet_ntoa(addr->sin_addr));
 
-	*res = mountlist_list();
+	if ((*res = mountlist_list()) == NULL)
+		xlog(L_WARNING, "dump request from %s failed.",
+			inet_ntoa(addr->sin_addr));
+
 	return 1;
 }
 
@@ -117,8 +134,14 @@ mount_umnt_1_svc(struct svc_req *rqstp, dirpath *argp, void *resp)
 	if (!(exp = auth_authenticate("unmount", sin, p))) {
 		return 1;
 	}
-	mountlist_del(exp, p);
-	export_reset (exp);
+	if (new_cache) {
+		if (strcmp(inet_ntoa(exp->m_client->m_addrlist[0]), exp->m_client->m_hostname))
+			mountlist_del(inet_ntoa(exp->m_client->m_addrlist[0]), exp->m_client->m_hostname);
+		mountlist_del(exp->m_client->m_hostname, p);
+	} else {
+		mountlist_del(exp->m_client->m_hostname, p);
+		export_reset (exp);
+	}
 	return 1;
 }
 
@@ -137,9 +160,11 @@ mount_export_1_svc(struct svc_req *rqstp, void *argp, exports *resp)
 {
 	struct sockaddr_in *addr =
 		(struct sockaddr_in *) svc_getcaller(rqstp->rq_xprt);
-	xlog(L_NOTICE, "export request from %s",
-		inet_ntoa(addr->sin_addr));
-	*resp = get_exportlist();
+
+	if ((*resp = get_exportlist()) == NULL)
+		xlog(L_WARNING, "export request from %s failed.",
+			inet_ntoa(addr->sin_addr));
+		
 	return 1;
 }
 
@@ -148,9 +173,10 @@ mount_exportall_1_svc(struct svc_req *rqstp, void *argp, exports *resp)
 {
 	struct sockaddr_in *addr =
 		(struct sockaddr_in *) svc_getcaller(rqstp->rq_xprt);
-	xlog(L_NOTICE, "exportall request from %s",
-		inet_ntoa(addr->sin_addr));
-	*resp = get_exportlist();
+
+	if ((*resp = get_exportlist()) == NULL)
+		xlog(L_WARNING, "exportall request from %s failed.",
+			inet_ntoa(addr->sin_addr));
 	return 1;
 }
 
@@ -222,7 +248,10 @@ mount_pathconf_2_svc(struct svc_req *rqstp, dirpath *path, ppathcnf *res)
 bool_t
 mount_mnt_3_svc(struct svc_req *rqstp, dirpath *path, mountres3 *res)
 {
-	static int	flavors[] = { AUTH_NULL, AUTH_UNIX };
+#define AUTH_GSS_KRB5 390003
+#define AUTH_GSS_KRB5I 390004
+#define AUTH_GSS_KRB5P 390005
+	static int	flavors[] = { AUTH_NULL, AUTH_UNIX, AUTH_GSS_KRB5, AUTH_GSS_KRB5I, AUTH_GSS_KRB5P};
 	struct nfs_fh_len *fh;
 
 	xlog(D_CALL, "MNT3(%s) called", *path);
@@ -231,7 +260,8 @@ mount_mnt_3_svc(struct svc_req *rqstp, dirpath *path, mountres3 *res)
 
 		ok->fhandle.fhandle3_len = fh->fh_size;
 		ok->fhandle.fhandle3_val = fh->fh_handle;
-		ok->auth_flavors.auth_flavors_len = 2;
+		ok->auth_flavors.auth_flavors_len
+			= sizeof(flavors)/sizeof(flavors[0]);
 		ok->auth_flavors.auth_flavors_val = flavors;
 	}
 	return 1;
@@ -277,7 +307,7 @@ get_rootfh(struct svc_req *rqstp, dirpath *path, int *error, int v3)
 		     p, strerror(errno));
 		*error = NFSERR_NOENT;
 	} else if (estb.st_dev != stb.st_dev
-		   /* && (!new_cache || !(exp->m_export.e_flags & NFSEXP_CROSSMNT)) */
+		   /* && (!new_cache || !(exp->m_export.e_flags & NFSEXP_CROSSMOUNT)) */
 		) {
 		xlog(L_WARNING, "request to export directory %s below nearest filesystem %s",
 		     p, exp->m_export.e_path);
@@ -322,7 +352,7 @@ get_rootfh(struct svc_req *rqstp, dirpath *path, int *error, int v3)
 						stb.st_dev, stb.st_ino);
 		}
 		if (fh != NULL) {
-			mountlist_add(exp, p);
+			mountlist_add(exp->m_client->m_hostname, p);
 			*error = NFS_OK;
 			export_reset (exp);
 			return fh;
@@ -397,11 +427,9 @@ get_exportlist(void)
 				xfree(c->gr_name);
 				xfree(c);
 				xfree (hp);
-				if ((c = *cp) == NULL)
-				  break;
+				continue;
 			      }
-			      else
-				xfree (hp);
+			      xfree (hp);
 			    }
 			    cp = &(c->gr_next);
 			  }
@@ -438,7 +466,7 @@ main(int argc, char **argv)
 
 	/* Parse the command line options and arguments. */
 	opterr = 0;
-	while ((c = getopt_long(argc, argv, "o:n:Fd:f:p:P:hN:V:v", longopts, NULL)) != EOF)
+	while ((c = getopt_long(argc, argv, "o:n:Fd:f:p:P:hH:N:V:v", longopts, NULL)) != EOF)
 		switch (c) {
 		case 'o':
 			descriptors = atoi(optarg);
@@ -457,6 +485,9 @@ main(int argc, char **argv)
 		case 'f':
 			export_file = optarg;
 			break;
+		case 'H': /* PRC: specify a high-availability callout program */
+			ha_callout_prog = optarg;
+			break;
 		case 'h':
 			usage(argv [0], 0);
 			break;
@@ -498,22 +529,26 @@ main(int argc, char **argv)
 		exit(1);
 	}
 
-	if (descriptors) {
-		if (getrlimit (RLIMIT_NOFILE, &rlim) != 0) {
-			fprintf(stderr, "%s: getrlimit (RLIMIT_NOFILE) failed: %s\n",
+	if (getrlimit (RLIMIT_NOFILE, &rlim) != 0)
+		fprintf(stderr, "%s: getrlimit (RLIMIT_NOFILE) failed: %s\n",
 				argv [0], strerror(errno));
-			exit(1);
-		}
-
-		rlim.rlim_cur = descriptors;
-		if (setrlimit (RLIMIT_NOFILE, &rlim) != 0) {
-			fprintf(stderr, "%s: setrlimit (RLIMIT_NOFILE) failed: %s\n",
-				argv [0], strerror(errno));
-			exit(1);
+	else {
+		/* glibc sunrpc code dies if getdtablesize > FD_SETSIZE */
+		if ((descriptors == 0 && rlim.rlim_cur > FD_SETSIZE) ||
+		    descriptors > FD_SETSIZE)
+			descriptors = FD_SETSIZE;
+		if (descriptors) {
+			rlim.rlim_cur = descriptors;
+			if (setrlimit (RLIMIT_NOFILE, &rlim) != 0) {
+				fprintf(stderr, "%s: setrlimit (RLIMIT_NOFILE) failed: %s\n",
+					argv [0], strerror(errno));
+				exit(1);
+			}
 		}
 	}
 	/* Initialize logging. */
-/*	xlog_open("mountd"); */
+	if (!foreground) xlog_stderr(0);
+	xlog_open("mountd");
 
 	sa.sa_handler = SIG_IGN;
 	sa.sa_flags = 0;
@@ -526,11 +561,8 @@ main(int argc, char **argv)
 	sigaction(SIGCHLD, &sa, NULL);
 
 	/* Daemons should close all extra filehandles ... *before* RPC init. */
-	if (!foreground) {
-		int fd = sysconf (_SC_OPEN_MAX);
-		while (--fd > 2)
-			(void) close(fd);
-	}
+	if (!foreground)
+		closeall(3);
 
 	new_cache = check_new_cache();
 	if (new_cache)
@@ -547,9 +579,10 @@ main(int argc, char **argv)
 			 mount_dispatch, port);
 
 	sa.sa_handler = killer;
-	sigaction(SIGHUP, &sa, NULL);
 	sigaction(SIGINT, &sa, NULL);
 	sigaction(SIGTERM, &sa, NULL);
+	sa.sa_handler = sig_hup;
+	sigaction(SIGHUP, &sa, NULL);
 
 	auth_init(export_file);
 
@@ -571,7 +604,6 @@ main(int argc, char **argv)
 			if (fd > 2) (void) close(fd);
 		}
 		setsid();
-		xlog_background();
 	}
 
 	my_svc_run();
@@ -587,6 +619,7 @@ usage(const char *prog, int n)
 "Usage: %s [-F|--foreground] [-h|--help] [-v|--version] [-d kind|--debug kind]\n"
 "	[-o num|--descriptors num] [-f exports-file|--exports-file=file]\n"
 "	[-p|--port port] [-V version|--nfs-version version]\n"
-"	[-N version|--no-nfs-version version] [-n|--no-tcp]\n", prog);
+"	[-N version|--no-nfs-version version] [-n|--no-tcp]\n"
+"	[-H ha-callout-prog]\n", prog);
 	exit(n);
 }
