@@ -12,7 +12,9 @@
  *	260295	 okir	started with simply syslog logging.
  */
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <syslog.h>
 #include <unistd.h>
@@ -32,11 +34,11 @@ static int	opt_debug = 0;	/* Will be command-line option, eventually */
 void log_init()
 {
 	if (!(run_mode & MODE_LOG_STDERR)) 
-		openlog(name_p, LOG_PID, LOG_DAEMON);
+		openlog(name_p, LOG_PID | LOG_NDELAY, LOG_DAEMON);
 
 	mypid = getpid();
 
-	log(L_WARNING,"Version %s Starting",version_p);
+	note(N_WARNING,"Version %s Starting",version_p);
 }
 
 void log_background(void)
@@ -54,7 +56,7 @@ void die(char *fmt, ...)
 	va_end(ap);
 	buffer[1023]=0;
 
-	log(L_FATAL, "%s", buffer);
+	note(N_FATAL, "%s", buffer);
 
 #ifndef DEBUG
 	exit (2);
@@ -63,7 +65,7 @@ void die(char *fmt, ...)
 #endif
 }
 
-void log(int level, char *fmt, ...)
+void note(int level, char *fmt, ...)
 {
 	char	buffer[1024];
 	va_list	ap;
@@ -73,7 +75,7 @@ void log(int level, char *fmt, ...)
 	va_end(ap);
 	buffer[1023]=0;
 
-	if ((!(run_mode & MODE_LOG_STDERR)) && (level < L_DEBUG)) {
+	if ((!(run_mode & MODE_LOG_STDERR)) && (level < N_DEBUG)) {
 		syslog(level, "%s", buffer);
 	} else if (run_mode & MODE_LOG_STDERR) {
 		/* Log everything, including dprintf() stuff to stderr */
